@@ -9,6 +9,8 @@ from simple_mq import SimpleMQ
 from django.contrib.auth.models import User
 from django.core.cache import cache
 from django.db import models
+from django.urls import reverse
+from django.utils.functional import classproperty
 from django.utils.translation import gettext_lazy as _
 from eveuniverse.helpers import meters_to_ly
 from eveuniverse.models import EveSolarSystem, EveType
@@ -251,7 +253,7 @@ class DiscordWebhook(models.Model):
                 "avatar_url": default_avatar_url(),
             }
             success = self.send_message_to_webhook(message)
-        except Exception as ex:
+        except OSError as ex:
             logger.warning(
                 "Failed to send test notification to webhook %s: %s",
                 self,
@@ -299,6 +301,12 @@ class Timer(models.Model):
         ANCHORING = "AN", _("Anchoring")
         UNANCHORING = "UA", _("Unanchoring")
         MOONMINING = "MM", _("Moon Mining")
+        PRELIMINARY = "PL", _("Preliminary")  # special timer with no date
+
+        @classproperty
+        def choices_for_notification_rules(cls) -> list:
+            """Subset of choices suitable for creating notification rules."""
+            return [choice for choice in cls.choices if choice[0] != cls.PRELIMINARY]
 
     class Objective(models.TextChoices):
         UNDEFINED = "UN", _("undefined")
@@ -313,6 +321,7 @@ class Timer(models.Model):
 
     date = models.DateTimeField(
         db_index=True,
+        null=True,
         help_text="Date when this timer happens",
     )
     details_image_url = models.CharField(
@@ -423,18 +432,27 @@ class Timer(models.Model):
             " of your organization"
         ),
     )
+    last_updated_at = models.DateTimeField(auto_now=True)
 
     objects = TimerManager()
 
     def __str__(self):
-        return "%s timer for %s @ %s" % (
+        return "{} timer for {}{}".format(
             self.get_timer_type_display(),
             self.structure_display_name,
-            self.date.strftime(DATETIME_FORMAT),
+            f" @ {self.date.strftime(DATETIME_FORMAT)}" if self.date else "",
+        )
+
+    def get_absolute_url(self) -> str:
+        url = reverse("structuretimers:timer_list")
+        return (
+            f"{url}?tab=preliminary"
+            if self.timer_type == self.Type.PRELIMINARY
+            else url
         )
 
     def save(self, *args, **kwargs):
-        """New save method for Timers. Will also schedule notifications for timers
+        """New save method for Timers. Will also schedule notifications for timers.
 
         Args:
             disable_notifications: Set to True to disable all notifications for the saved timer
@@ -443,14 +461,17 @@ class Timer(models.Model):
             disable_notifications = kwargs.pop("disable_notifications")
         except KeyError:
             disable_notifications = False
-        notification_enabled = (
-            STRUCTURETIMERS_NOTIFICATIONS_ENABLED and not disable_notifications
+        schedule_notifications = (
+            STRUCTURETIMERS_NOTIFICATIONS_ENABLED
+            and not disable_notifications
+            and self.timer_type != self.Type.PRELIMINARY
         )
         try:
             old_instance = Timer.objects.get(pk=self.pk)
         except (Timer.DoesNotExist, ValueError):
             needs_recalc = True
             date_changed = False
+            old_instance = None
         else:
             needs_recalc = self.eve_solar_system != old_instance.eve_solar_system
             date_changed = self.date != old_instance.date
@@ -461,7 +482,13 @@ class Timer(models.Model):
             _task_calc_timer_distances_for_all_staging_systems().apply_async(
                 args=[self.pk], priority=4
             )
-        if notification_enabled and (is_new or date_changed):
+        if (
+            self.timer_type == self.Type.PRELIMINARY
+            and old_instance
+            and old_instance.timer_type != self.Type.PRELIMINARY
+        ):
+            self.scheduled_notifications.all().delete()
+        if schedule_notifications and (is_new or date_changed):
             _task_schedule_notifications_for_timer().apply_async(
                 kwargs={"timer_pk": self.pk, "is_new": is_new}, priority=3
             )
@@ -683,7 +710,7 @@ class NotificationRule(models.Model):
         help_text="whether this rule is currently active",
     )
     require_timer_types = MultiSelectField(
-        choices=Timer.Type.choices,
+        choices=Timer.Type.choices_for_notification_rules,
         blank=True,
         help_text=(
             "Timer must have one of the given timer types "
@@ -691,7 +718,7 @@ class NotificationRule(models.Model):
         ),
     )
     exclude_timer_types = MultiSelectField(
-        choices=Timer.Type.choices,
+        choices=Timer.Type.choices_for_notification_rules,
         blank=True,
         help_text="Timer must NOT have one of the given timer types",
     )
@@ -798,6 +825,8 @@ class NotificationRule(models.Model):
 
     def is_matching_timer(self, timer: "Timer") -> bool:
         """returns True if notification rule is matching the given timer, else False"""
+        if timer.date is None:
+            return False
         is_matching = True
         if is_matching and self.is_important == self.Clause.REQUIRED:
             is_matching = timer.is_important
