@@ -1,43 +1,24 @@
 from datetime import timedelta
 from unittest.mock import Mock, patch
 
-from requests.exceptions import ConnectionError as NewConnectionError
-from requests.exceptions import HTTPError
-
-from django.test import TestCase
-from django.test.utils import override_settings
+from django.test import TestCase, override_settings
 from django.urls import reverse
 from django.utils.timezone import now
 from django_webtest import WebTest
 
 from allianceauth.tests.auth_utils import AuthUtils
 
-from ..models import DiscordWebhook, Timer
+from ..models import DiscordWebhook, ScheduledNotification, Timer
 from ..tasks import send_test_message_to_webhook
-from . import (
-    LoadTestDataMixin,
-    add_permission_to_user_by_name,
-    create_fake_timer,
-    create_test_user,
-)
-from .testdata import test_data_filename, test_image_filename
+from .testdata.factory import create_notification_rule, create_timer, create_user
+from .testdata.fixtures import LoadTestDataMixin
+from .utils import add_permission_to_user_by_name
 
 MODELS_PATH = "structuretimers.models"
 FORMS_PATH = "structuretimers.forms"
 TASKS_PATH = "structuretimers.tasks"
 
 
-def bytes_from_file(filename, chunksize=8192):
-    with open(filename, "rb") as f:
-        while True:
-            chunk = f.read(chunksize)
-            if chunk:
-                for b in chunk:
-                    yield b
-            else:
-                break
-
-
 @patch(MODELS_PATH + "._task_calc_timer_distances_for_all_staging_systems", Mock())
 @patch(MODELS_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
 class TestUI(LoadTestDataMixin, WebTest):
@@ -46,17 +27,17 @@ class TestUI(LoadTestDataMixin, WebTest):
         super().setUpClass()
 
         # user 1
-        cls.user_1 = create_test_user(cls.character_1)
+        cls.user_1 = create_user(cls.character_1)
 
         # user 2
-        cls.user_2 = create_test_user(cls.character_2)
+        cls.user_2 = create_user(cls.character_2)
         cls.user_2 = add_permission_to_user_by_name(
             "structuretimers.create_timer", cls.user_2
         )
 
     @patch(MODELS_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self) -> None:
-        self.timer_1 = create_fake_timer(
+        self.timer_1 = create_timer(
             structure_name="Timer 1",
             date=now() + timedelta(hours=4),
             eve_character=self.character_2,
@@ -65,7 +46,7 @@ class TestUI(LoadTestDataMixin, WebTest):
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
         )
-        self.timer_2 = create_fake_timer(
+        self.timer_2 = create_timer(
             structure_name="Timer 2",
             date=now() - timedelta(hours=8),
             eve_character=self.character_2,
@@ -74,7 +55,7 @@ class TestUI(LoadTestDataMixin, WebTest):
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
         )
-        self.timer_3 = create_fake_timer(
+        self.timer_3 = create_timer(
             structure_name="Timer 3",
             date=now() - timedelta(hours=8),
             eve_character=self.character_2,
@@ -145,206 +126,6 @@ class TestUI(LoadTestDataMixin, WebTest):
         self.assertEqual(response.status_code, 302)
         self.assertEqual(response.url, reverse("authentication:dashboard"))
 
-    def test_should_not_allow_submitting_without_structure_type(self):
-        # login
-        self.app.set_user(self.user_2)
-
-        # user clicks on "Add Timer"
-        add_timer = self.app.get(reverse("structuretimers:add"))
-        self.assertEqual(add_timer.status_code, 200)
-
-        # user enters data and clicks create
-        form = add_timer.forms["add-timer-form"]
-        form["structure_name"] = "Timer 4"
-        form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
-        form["days_left"] = -1
-        form["hours_left"] = 2
-        form["minutes_left"] = 3
-        response = form.submit()
-
-        # assert results
-        self.assertEqual(response.status_code, 200)
-        self.assertIn("correct the input errors", response.text)
-
-    def test_should_not_allow_submitting_without_solar_system(self):
-        # login
-        self.app.set_user(self.user_2)
-
-        # user clicks on "Add Timer"
-        add_timer = self.app.get(reverse("structuretimers:add"))
-        self.assertEqual(add_timer.status_code, 200)
-
-        # user enters data and clicks create
-        form = add_timer.forms["add-timer-form"]
-        form["structure_name"] = "Timer 4"
-        form["structure_type_2"].force_value([str(self.type_astrahus.id)])
-        form["days_left"] = -1
-        form["hours_left"] = 2
-        form["minutes_left"] = 3
-        response = form.submit()
-
-        # assert results
-        self.assertEqual(response.status_code, 200)
-        self.assertIn("correct the input errors", response.text)
-
-    def test_should_not_allow_entering_invalid_days(self):
-        # login
-        self.app.set_user(self.user_2)
-
-        # user clicks on "Add Timer"
-        add_timer = self.app.get(reverse("structuretimers:add"))
-        self.assertEqual(add_timer.status_code, 200)
-
-        # user enters data and clicks create
-        form = add_timer.forms["add-timer-form"]
-        form["structure_name"] = "Timer 4"
-        form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
-        form["structure_type_2"].force_value([str(self.type_astrahus.id)])
-        form["days_left"] = -1
-        form["hours_left"] = 2
-        form["minutes_left"] = 3
-        response = form.submit()
-
-        # assert results
-        self.assertEqual(response.status_code, 200)
-        self.assertIn("correct the input errors", response.text)
-
-    @patch(FORMS_PATH + ".requests.get", spec=True)
-    def test_should_show_error_when_image_can_not_be_loaded_1(self, mock_get):
-        mock_get.side_effect = NewConnectionError
-
-        # login
-        self.app.set_user(self.user_2)
-
-        # user clicks on "Add Timer"
-        add_timer = self.app.get(reverse("structuretimers:add"))
-        self.assertEqual(add_timer.status_code, 200)
-
-        # user enters data and clicks create
-        form = add_timer.forms["add-timer-form"]
-        form["structure_name"] = "Timer 4"
-        form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
-        form["structure_type_2"].force_value([str(self.type_astrahus.id)])
-        form["days_left"] = 1
-        form["hours_left"] = 2
-        form["minutes_left"] = 3
-        form["details_image_url"] = "http://www.example.com/image.png"
-        response = form.submit()
-
-        # assert results
-        self.assertEqual(response.status_code, 200)
-        self.assertIn("correct the input errors", response.text)
-
-    @patch(FORMS_PATH + ".requests.get", spec=True)
-    def test_should_show_error_when_image_can_not_be_loaded_2(self, mock_get):
-        """
-        when user entered invalid day
-        then page can not be submitted and error is shown to user
-        """
-        mock_get.side_effect = HTTPError
-
-        # login
-        self.app.set_user(self.user_2)
-
-        # user clicks on "Add Timer"
-        add_timer = self.app.get(reverse("structuretimers:add"))
-        self.assertEqual(add_timer.status_code, 200)
-
-        # user enters data and clicks create
-        form = add_timer.forms["add-timer-form"]
-        form["days_left"] = 1
-        form["structure_name"] = "Timer 4"
-        form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
-        form["structure_type_2"].force_value([str(self.type_astrahus.id)])
-        form["days_left"] = 1
-        form["hours_left"] = 2
-        form["minutes_left"] = 3
-        form["details_image_url"] = "http://www.example.com/image.png"
-        response = form.submit()
-
-        # assert results
-        self.assertEqual(response.status_code, 200)
-        self.assertIn("correct the input errors", response.text)
-
-    @patch(FORMS_PATH + ".requests.get", spec=True)
-    def test_should_create_timer_with_valid_details_image(self, mock_get):
-        image_file = bytearray(bytes_from_file(test_image_filename()))
-        mock_get.return_value.content = image_file
-
-        # login
-        self.app.set_user(self.user_2)
-
-        # user clicks on "Add Timer"
-        add_timer = self.app.get(reverse("structuretimers:add"))
-        self.assertEqual(add_timer.status_code, 200)
-
-        # user enters data and clicks create
-        form = add_timer.forms["add-timer-form"]
-        form["days_left"] = 1
-        form["structure_name"] = "Timer 4"
-        form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
-        form["structure_type_2"].force_value([str(self.type_astrahus.id)])
-        form["days_left"] = 1
-        form["hours_left"] = 2
-        form["minutes_left"] = 3
-        form["details_image_url"] = "http://www.example.com/image.png"
-        response = form.submit()
-
-        # assert results
-        self.assertEqual(response.status_code, 302)
-        self.assertEqual(response.url, reverse("structuretimers:timer_list"))
-        self.assertTrue(Timer.objects.filter(structure_name="Timer 4").exists())
-
-    @patch(FORMS_PATH + ".requests.get", spec=True)
-    def test_should_not_allow_invalid_link_for_detail_images(self, mock_get):
-        """
-        when user provides invalid file link
-        then page can not be submitted and error is shown to user
-        """
-        image_file = bytearray(bytes_from_file(test_data_filename()))
-        mock_get.return_value.content = image_file
-
-        # login
-        self.app.set_user(self.user_2)
-
-        # user clicks on "Add Timer"
-        add_timer = self.app.get(reverse("structuretimers:add"))
-        self.assertEqual(add_timer.status_code, 200)
-
-        # user enters data and clicks create
-        form = add_timer.forms["add-timer-form"]
-        form["days_left"] = 1
-        form["structure_name"] = "Timer 4"
-        form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
-        form["structure_type_2"].force_value([str(self.type_astrahus.id)])
-        form["days_left"] = 1
-        form["hours_left"] = 2
-        form["minutes_left"] = 3
-        form["details_image_url"] = "http://www.example.com/image.png"
-        response = form.submit()
-
-        # assert results
-        self.assertEqual(response.status_code, 200)
-        self.assertIn("correct the input errors", response.text)
-
-    def test_should_not_allow_moon_mining_type_for_non_mining_structures(self):
-        # given
-        self.app.set_user(self.user_2)
-        add_timer = self.app.get(reverse("structuretimers:add"))
-        # when
-        form = add_timer.forms["add-timer-form"]
-        form["structure_name"] = "Timer 4"
-        form["eve_solar_system_2"].force_value([str(self.system_abune.id)])
-        form["structure_type_2"].force_value([str(self.type_astrahus.id)])
-        form["timer_type"] = Timer.Type.MOONMINING
-        form["days_left"] = 1
-        form["hours_left"] = 2
-        form["minutes_left"] = 3
-        response = form.submit()
-        # then
-        self.assertEqual(response.status_code, 200)
-        self.assertIn("correct the input errors", response.text)
-
     def test_edit_existing_timer(self):
         """
         when user has permissions
@@ -398,7 +179,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         """
 
         # login
-        user_3 = create_test_user(self.character_3)
+        user_3 = create_user(self.character_3)
         user_3 = add_permission_to_user_by_name("structuretimers.create_timer", user_3)
         self.app.set_user(user_3)
 
@@ -414,7 +195,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         """
 
         # login
-        user_3 = create_test_user(self.character_3)
+        user_3 = create_user(self.character_3)
         user_3 = add_permission_to_user_by_name("structuretimers.manage_timer", user_3)
         self.app.set_user(user_3)
 
@@ -450,7 +231,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         self.timer_3.save()
 
         # login
-        user_3 = create_test_user(self.character_3)
+        user_3 = create_user(self.character_3)
         user_3 = add_permission_to_user_by_name("structuretimers.create_timer", user_3)
         user_3 = add_permission_to_user_by_name("structuretimers.manage_timer", user_3)
         self.app.set_user(user_3)
@@ -470,7 +251,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         self.timer_3.save()
 
         # login
-        user_3 = create_test_user(self.character_3)
+        user_3 = create_user(self.character_3)
         user_3 = add_permission_to_user_by_name("structuretimers.create_timer", user_3)
         user_3 = add_permission_to_user_by_name("structuretimers.manage_timer", user_3)
         self.app.set_user(user_3)
@@ -487,7 +268,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         """
 
         # login
-        user_3 = create_test_user(self.character_3)
+        user_3 = create_user(self.character_3)
         user_3 = add_permission_to_user_by_name("structuretimers.manage_timer", user_3)
         self.app.set_user(user_3)
 
@@ -547,7 +328,7 @@ class TestUI(LoadTestDataMixin, WebTest):
         """
 
         # login
-        user_3 = create_test_user(self.character_3)
+        user_3 = create_user(self.character_3)
         user_3 = add_permission_to_user_by_name("structuretimers.create_timer", user_3)
         self.app.set_user(user_3)
 
@@ -571,7 +352,7 @@ class TestSendNotifications(LoadTestDataMixin, TestCase):
         self.rule.webhooks.add(self.webhook)
 
     def test_normal(self, mock_execute):
-        create_fake_timer(
+        create_timer(
             structure_name="Test_1",
             eve_solar_system=self.system_abune,
             structure_type=self.type_raitaru,
@@ -604,3 +385,26 @@ class TestTestMessageToWebhook(LoadTestDataMixin, TestCase):
         )
         self.assertEqual(mock_execute.call_count, 1)
         self.assertTrue(mock_notify.called)
+
+
+@override_settings(CELERY_ALWAYS_EAGER=True, CELERY_EAGER_PROPAGATES_EXCEPTIONS=True)
+@patch(MODELS_PATH + "._task_calc_timer_distances_for_all_staging_systems", Mock())
+class TestTimerSave(LoadTestDataMixin, TestCase):
+    @classmethod
+    def setUpClass(cls) -> None:
+        super().setUpClass()
+        cls.webhook = DiscordWebhook.objects.create(
+            name="Dummy", url="http://www.example.com"
+        )
+
+    def test_schedule_notifications_for_new_timers_2(self):
+        # when
+        create_notification_rule()
+        timer = create_timer(
+            date=now() + timedelta(hours=4),
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_astrahus,
+            enabled_notifications=True,
+        )
+        # then
+        self.assertTrue(ScheduledNotification.objects.filter(timer=timer).exists())
