@@ -13,13 +13,9 @@ from app_utils.testing import (
 )
 
 from ..models import Timer
-from . import (
-    LoadTestDataMixin,
-    add_permission_to_user_by_name,
-    create_fake_staging_system,
-    create_fake_timer,
-    create_test_user,
-)
+from .testdata.factory import create_staging_system, create_timer, create_user
+from .testdata.fixtures import LoadTestDataMixin
+from .utils import add_permission_to_user_by_name
 
 MODELS_PATH = "structuretimers.models"
 
@@ -29,15 +25,15 @@ class TestViewBase(LoadTestDataMixin, TestCase):
     @patch(MODELS_PATH + ".STRUCTURETIMERS_NOTIFICATIONS_ENABLED", False)
     def setUp(self):
         # user
-        self.user_1 = create_test_user(self.character_1)
-        self.user_2 = create_test_user(self.character_2)
+        self.user_1 = create_user(self.character_1)
+        self.user_2 = create_user(self.character_2)
         self.user_2 = add_permission_to_user_by_name(
             "structuretimers.manage_timer", self.user_2
         )
-        self.user_3 = create_test_user(self.character_3)
+        self.user_3 = create_user(self.character_3)
 
         # timers
-        self.timer_1 = create_fake_timer(
+        self.timer_1 = create_timer(
             structure_name="Timer 1",
             location_details="Near the star",
             date=now() + timedelta(hours=4),
@@ -50,7 +46,7 @@ class TestViewBase(LoadTestDataMixin, TestCase):
             details_image_url="http://www.example.com/dummy.png",
             details_notes="Some notes",
         )
-        self.timer_2 = create_fake_timer(
+        self.timer_2 = create_timer(
             structure_name="Timer 2",
             date=now() - timedelta(hours=8),
             eve_character=self.character_1,
@@ -60,7 +56,7 @@ class TestViewBase(LoadTestDataMixin, TestCase):
             structure_type=self.type_raitaru,
             is_important=True,
         )
-        self.timer_3 = create_fake_timer(
+        self.timer_3 = create_timer(
             structure_name="Timer 3",
             date=now() - timedelta(hours=8),
             eve_character=self.character_1,
@@ -69,10 +65,17 @@ class TestViewBase(LoadTestDataMixin, TestCase):
             eve_solar_system=self.system_enaluri,
             structure_type=self.type_astrahus,
         )
+        self.timer_4 = create_timer(
+            timer_type=Timer.Type.PRELIMINARY,
+            structure_name="Timer 4",
+            eve_character=self.character_1,
+            eve_corporation=self.corporation_1,
+            user=self.user_1,
+        )
 
 
 class TestListData(TestViewBase):
-    def _timer_list_data(self, tab_name: str = "current", user: User = None):
+    def _get_timer_list_data(self, tab_name: str = "current", user: User = None):
         if not user:
             user = self.user_1
         self.client.force_login(user)
@@ -80,8 +83,10 @@ class TestListData(TestViewBase):
             reverse("structuretimers:timer_list_data", args=[tab_name])
         )
 
-    def _timer_list_data_ids(self, tab_name: str = "current", user: User = None) -> set:
-        response = self._timer_list_data(tab_name, user)
+    def _get_timer_list_data_ids(
+        self, tab_name: str = "current", user: User = None
+    ) -> set:
+        response = self._get_timer_list_data(tab_name, user)
         self.assertEqual(response.status_code, 200)
         return set(json_response_to_dict(response).keys())
 
@@ -93,16 +98,22 @@ class TestListData(TestViewBase):
 
     def test_return_current_timers(self):
         # when
-        timer_ids = self._timer_list_data_ids("current")
+        timer_ids = self._get_timer_list_data_ids("current")
         # then
         self.assertSetEqual(timer_ids, {self.timer_1.id})
 
     def test_return_past_timers(self):
         # when
-        timer_ids = self._timer_list_data_ids("past")
+        timer_ids = self._get_timer_list_data_ids("past")
         # then
         self.assertSetEqual(timer_ids, {self.timer_2.id, self.timer_3.id})
 
+    def test_return_preliminary_timers(self):
+        # when
+        timer_ids = self._get_timer_list_data_ids("preliminary")
+        # then
+        self.assertSetEqual(timer_ids, {self.timer_4.id})
+
     def test_should_require_permission(self):
         # given
         user, _ = create_user_from_evecharacter(1003)
@@ -115,7 +126,7 @@ class TestListData(TestViewBase):
         self.assertEqual(response.status_code, 302)
 
     def test_show_corp_restricted_to_corp_member(self):
-        timer_4 = create_fake_timer(
+        timer_4 = create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -125,12 +136,12 @@ class TestListData(TestViewBase):
             user=self.user_2,
             visibility=Timer.Visibility.CORPORATION,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id, timer_4.id}
         self.assertSetEqual(timer_ids, expected)
 
     def test_dont_show_corp_restricted_to_non_corp_member(self):
-        create_fake_timer(
+        create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -140,12 +151,12 @@ class TestListData(TestViewBase):
             user=self.user_3,
             visibility=Timer.Visibility.CORPORATION,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id}
         self.assertSetEqual(timer_ids, expected)
 
     def test_show_alliance_restricted_to_alliance_member(self):
-        timer_4 = create_fake_timer(
+        timer_4 = create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -156,12 +167,12 @@ class TestListData(TestViewBase):
             user=self.user_2,
             visibility=Timer.Visibility.ALLIANCE,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id, timer_4.id}
         self.assertSetEqual(timer_ids, expected)
 
     def test_dont_show_alliance_restricted_to_non_alliance_member(self):
-        create_fake_timer(
+        create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -172,7 +183,7 @@ class TestListData(TestViewBase):
             user=self.user_3,
             visibility=Timer.Visibility.ALLIANCE,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id}
         self.assertSetEqual(timer_ids, expected)
 
@@ -180,7 +191,7 @@ class TestListData(TestViewBase):
         self.user_1 = add_permission_to_user_by_name(
             "structuretimers.opsec_access", self.user_1
         )
-        timer_4 = create_fake_timer(
+        timer_4 = create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -190,12 +201,12 @@ class TestListData(TestViewBase):
             user=self.user_3,
             is_opsec=True,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id, timer_4.id}
         self.assertSetEqual(timer_ids, expected)
 
     def test_dont_show_opsec_restricted_to_non_opsec_member(self):
-        create_fake_timer(
+        create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -205,7 +216,7 @@ class TestListData(TestViewBase):
             user=self.user_3,
             is_opsec=True,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id}
         self.assertSetEqual(timer_ids, expected)
 
@@ -213,7 +224,7 @@ class TestListData(TestViewBase):
         self.user_1 = add_permission_to_user_by_name(
             "structuretimers.opsec_access", self.user_1
         )
-        create_fake_timer(
+        create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -224,12 +235,12 @@ class TestListData(TestViewBase):
             is_opsec=True,
             visibility=Timer.Visibility.CORPORATION,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id}
         self.assertSetEqual(timer_ids, expected)
 
     def test_show_corp_timer_to_creator_of_different_corp(self):
-        timer_4 = create_fake_timer(
+        timer_4 = create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -239,12 +250,12 @@ class TestListData(TestViewBase):
             visibility=Timer.Visibility.CORPORATION,
             user=self.user_1,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id, timer_4.id}
         self.assertSetEqual(timer_ids, expected)
 
     def test_show_alliance_timer_to_creator_of_different_alliance(self):
-        timer_4 = create_fake_timer(
+        timer_4 = create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -255,29 +266,29 @@ class TestListData(TestViewBase):
             visibility=Timer.Visibility.ALLIANCE,
             user=self.user_1,
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id, timer_4.id}
         self.assertSetEqual(timer_ids, expected)
 
     def test_can_show_timers_without_user_character_corporation_or_alliance(self):
-        timer_4 = create_fake_timer(
+        timer_4 = create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
             date=now() + timedelta(hours=8),
         )
-        timer_ids = self._timer_list_data_ids()
+        timer_ids = self._get_timer_list_data_ids()
         expected = {self.timer_1.id, timer_4.id}
         self.assertSetEqual(timer_ids, expected)
 
     def test_list_for_manager(self):
-        timer_ids = self._timer_list_data_ids(user=self.user_2)
+        timer_ids = self._get_timer_list_data_ids(user=self.user_2)
         expected = {self.timer_1.id}
         self.assertSetEqual(timer_ids, expected)
 
-    def test_should_include_distances(self):
+    def test_should_include_distances_1(self):
         # given
-        timer = create_fake_timer(
+        timer = create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -287,7 +298,7 @@ class TestListData(TestViewBase):
             eve_corporation=self.corporation_3,
             user=self.user_1,
         )
-        staging_system = create_fake_staging_system(
+        staging_system = create_staging_system(
             eve_solar_system=self.system_enaluri,
             light_years=1.2,
             jumps=3,
@@ -305,9 +316,9 @@ class TestListData(TestViewBase):
         self.assertEqual(obj["distance_jumps"], 3)
         self.assertTrue(obj["distance"])
 
-    def test_should_not_include_distances(self):
+    def test_should_not_include_distances_1(self):
         # given
-        timer = create_fake_timer(
+        timer = create_timer(
             structure_name="Timer 4",
             eve_solar_system=self.system_abune,
             structure_type=self.type_astrahus,
@@ -317,11 +328,35 @@ class TestListData(TestViewBase):
             eve_corporation=self.corporation_3,
             user=self.user_1,
         )
-        staging_system = create_fake_staging_system(
-            eve_solar_system=self.system_enaluri
+        create_staging_system(
+            eve_solar_system=self.system_enaluri, light_years=1.2, jumps=3, is_main=True
         )
         self.client.force_login(self.user_1)
         # when
+        response = self.client.get(
+            reverse("structuretimers:timer_list_data", args=["current"])
+        )
+        # then
+        data = json_response_to_dict(response)
+        obj = data[timer.id]
+        self.assertIsNone(obj["distance_light_years"])
+        self.assertIsNone(obj["distance_jumps"])
+
+    def test_should_not_include_distances_2(self):
+        # given
+        timer = create_timer(
+            structure_name="Timer 4",
+            eve_solar_system=self.system_abune,
+            structure_type=self.type_astrahus,
+            date=now() + timedelta(hours=8),
+            eve_character=self.character_3,
+            eve_alliance=self.alliance_3,
+            eve_corporation=self.corporation_3,
+            user=self.user_1,
+        )
+        staging_system = create_staging_system(eve_solar_system=self.system_enaluri)
+        self.client.force_login(self.user_1)
+        # when
         response = self.client.get(
             reverse("structuretimers:timer_list_data", args=["current"])
             + f"?staging={staging_system.pk}"
@@ -334,21 +369,28 @@ class TestListData(TestViewBase):
 
 
 @patch(MODELS_PATH + "._task_calc_timer_distances_for_all_staging_systems", Mock())
-class TestGetTimerData(TestViewBase):
-    def test_should_return_timer(self):
+class TestDetailView(TestViewBase):
+    def test_should_return_normal_timer(self):
         # given
         self.client.force_login(self.user_1)
         # when
         response = self.client.get(
-            reverse("structuretimers:get_timer_data", args=[self.timer_1.pk])
+            reverse("structuretimers:detail", args=[self.timer_1.pk])
         )
         # then
         self.assertEqual(response.status_code, 200)
-        data = json_response_to_python(response)
-        self.assertEqual(
-            data["structure_display_name"],
-            'Astrahus "Timer 1" in Abune near Near the star',
+        self.assertIn("Timer 1", response.rendered_content)
+
+    def test_should_return_preliminary_timer(self):
+        # given
+        self.client.force_login(self.user_1)
+        # when
+        response = self.client.get(
+            reverse("structuretimers:detail", args=[self.timer_4.pk])
         )
+        # then
+        self.assertEqual(response.status_code, 200)
+        self.assertIn("Timer 4", response.rendered_content)
 
     def test_forbidden(self):
         # given
@@ -357,7 +399,7 @@ class TestGetTimerData(TestViewBase):
         self.timer_1.save()
         # when
         response = self.client.get(
-            reverse("structuretimers:get_timer_data", args=[self.timer_1.pk])
+            reverse("structuretimers:detail", args=[self.timer_1.pk])
         )
         # then
         self.assertEqual(response.status_code, 302)
@@ -367,7 +409,7 @@ class TestSelect2Views(LoadTestDataMixin, TestCase):
     @classmethod
     def setUpClass(cls) -> None:
         super().setUpClass()
-        cls.user_1 = create_test_user(cls.character_1)
+        cls.user_1 = create_user(cls.character_1)
 
     def test_should_return_solar_systems(self):
         # given
