@@ -1,11 +1,11 @@
 import math
 from copy import deepcopy
-from datetime import timedelta
 
+from django.contrib import messages
 from django.contrib.auth.mixins import LoginRequiredMixin, PermissionRequiredMixin
 from django.core.exceptions import PermissionDenied
-from django.shortcuts import get_object_or_404
-from django.urls import reverse, reverse_lazy
+from django.shortcuts import HttpResponse, get_object_or_404
+from django.urls import reverse
 from django.utils.html import format_html
 from django.utils.safestring import mark_safe
 from django.utils.timezone import now
@@ -14,17 +14,16 @@ from django.views import View
 from django.views.generic import (
     CreateView,
     DeleteView,
+    DetailView,
     ListView,
     TemplateView,
     UpdateView,
 )
-from django.views.generic.detail import BaseDetailView
 from eveuniverse.models import EveSolarSystem, EveType
 
 from allianceauth.eveonline.evelinks import dotlan
 from allianceauth.services.hooks import get_extension_logger
 from app_utils.logging import LoggerAddTag
-from app_utils.messages import messages_plus
 from app_utils.views import (
     JSONResponseMixin,
     bootstrap_label_html,
@@ -39,14 +38,7 @@ from .app_settings import (
     STRUCTURETIMERS_DEFAULT_PAGE_LENGTH,
     STRUCTURETIMERS_PAGING_ENABLED,
 )
-from .constants import (
-    EVE_CATEGORY_ID_STRUCTURE,
-    EVE_GROUP_ID_CONTROL_TOWER,
-    EVE_GROUP_ID_MOBILE_DEPOT,
-    EVE_TYPE_ID_CUSTOMS_OFFICE,
-    EVE_TYPE_ID_IHUB,
-    EVE_TYPE_ID_TCU,
-)
+from .constants import EveCategoryId, EveGroupId, EveTypeId
 from .forms import TimerForm
 from .models import DistancesFromStaging, StagingSystem, Timer
 
@@ -82,6 +74,7 @@ class TimerListView(LoginRequiredMixin, PermissionRequiredMixin, TemplateView):
                 "data_tables_paging": STRUCTURETIMERS_PAGING_ENABLED,
                 "selected_staging_system": selected_staging_system,
                 "stageing_systems": stageing_systems,
+                "tab": self.request.GET.get("tab", "current"),
             }
         )
         return context
@@ -101,14 +94,9 @@ class TimerListDataView(
     def get_queryset(self):
         qs = super().get_queryset()
         timers_qs = qs.visible_to_user(self.request.user)
-        tab_name = self.kwargs.get("tab_name")
-        if tab_name == "current":
-            timers_qs = timers_qs.filter(
-                date__gte=now() - timedelta(hours=MAX_HOURS_PASSED)
-            )
-        else:
-            timers_qs = timers_qs.filter(date__lt=now())
-
+        timers_qs = timers_qs.filter_by_tab(
+            tab_name=self.kwargs.get("tab_name"), max_hours_passed=MAX_HOURS_PASSED
+        )
         timers_qs = timers_qs.select_related(
             "eve_solar_system__eve_constellation__eve_region",
             "structure_type",
@@ -247,21 +235,22 @@ class TimerListDataView(
                 visibility = timer.eve_alliance.alliance_name
             elif timer.visibility == Timer.Visibility.CORPORATION:
                 visibility = corporation_name
-
+            distances_light_years = distances.light_years if distances else None
             data.append(
                 {
                     "id": timer.id,
-                    "local_time": timer.date.isoformat(),
-                    "date": timer.date.isoformat(),
+                    "local_time": timer.date.isoformat() if timer.date else "",
+                    "date": timer.date.isoformat() if timer.date else "",
                     "location": location,
                     "structure_details": structure,
                     "name_objective": name,
                     "owner": objective,
                     # "creator": creator,
-                    "distance": distance_text,
-                    "distance_light_years": distances.light_years
-                    if distances
-                    else None,
+                    "distance": {
+                        "display": distance_text,
+                        "sort": distances_light_years,
+                    },
+                    "distance_light_years": distances_light_years,
                     "distance_jumps": distances.jumps if distances else None,
                     "actions": self._get_data_actions(timer),
                     "timer_type_name": timer.get_timer_type_display(),
@@ -273,9 +262,10 @@ class TimerListDataView(
                     "visibility": visibility,
                     "opsec_str": yesno_str(timer.is_opsec),
                     "is_opsec": timer.is_opsec,
-                    "is_passed": timer.date < now(),
+                    "is_passed": timer.date < now() if timer.date else None,
                     "is_important": timer.is_important,
                     "is_restricted": is_restricted,
+                    "last_updated_at": timer.last_updated_at.isoformat(),
                 }
             )
 
@@ -329,37 +319,18 @@ class TimerListDataView(
         return no_wrap_html(actions)
 
 
-class TimerDetailDataView(
-    LoginRequiredMixin, PermissionRequiredMixin, JSONResponseMixin, BaseDetailView
-):
+class TimerDetailDataView(LoginRequiredMixin, PermissionRequiredMixin, DetailView):
     permission_required = "structuretimers.basic_access"
     model = Timer
 
-    def render_to_response(self, context, **response_kwargs):
-        return self.render_to_json_response(context, **response_kwargs)
-
     def get_queryset(self):
         qs = super().get_queryset()
         return qs.visible_to_user(self.request.user).select_related(
             "structure_type", "eve_solar_system"
         )
 
-    def get_context_data(self, object):
-        context = {
-            "display_name": str(object),
-            "structure_display_name": object.structure_display_name,
-            "date": object.date.strftime(DATETIME_FORMAT),
-            "details_image_url": object.details_image_url
-            if object.details_image_url
-            else "",
-            "notes": object.details_notes if object.details_notes else "",
-        }
-        return context
-
 
 class TimerManagementView(LoginRequiredMixin, PermissionRequiredMixin, View):
-    index_redirect = "structuretimers:timer_list"
-    success_url = reverse_lazy(index_redirect)
     model = Timer
     form_class = TimerForm
     title = _("Edit Structure Timer")
@@ -369,12 +340,15 @@ class TimerManagementView(LoginRequiredMixin, PermissionRequiredMixin, View):
         context["title"] = self.title
         return context
 
+    def send_success_message(self, keyword: str) -> None:
+        """Inform user about result of his action via message."""
+        timer = self.object
+        messages.info(self.request, f"{keyword}: {timer}.")
+
 
 class AddUpdateMixin:
     def get_form_kwargs(self):
-        """
-        Inject the request user into the kwargs passed to the form
-        """
+        """Inject the request user into the kwargs passed to the form."""
         kwargs = super().get_form_kwargs()
         kwargs.update({"user": self.request.user})
         return kwargs
@@ -397,15 +371,7 @@ class CreateTimerView(TimerManagementView, AddUpdateMixin, CreateView):
             timer.date,
             self.request.user,
         )
-        messages_plus.success(
-            self.request,
-            _("Added new timer for %(type)s in %(system)s at %(time)s.")
-            % {
-                "type": timer.structure_type.name,
-                "system": timer.eve_solar_system.name,
-                "time": timer.date.strftime(DATETIME_FORMAT),
-            },
-        )
+        self.send_success_message(_("Added"))
         return result
 
 
@@ -429,6 +395,11 @@ class EditTimerMixin:
 class EditTimerView(EditTimerMixin, TimerManagementView, AddUpdateMixin, UpdateView):
     template_name_suffix = "_update_form"
 
+    def form_valid(self, form):
+        result = super().form_valid(form)
+        self.send_success_message(_("Updated"))
+        return result
+
 
 class CopyTimerView(CreateTimerView):
     def get_form_kwargs(self):
@@ -442,64 +413,78 @@ class CopyTimerView(CreateTimerView):
 
 
 class RemoveTimerView(EditTimerMixin, TimerManagementView, DeleteView):
-    pass
+    def __init__(self) -> None:
+        self.object = None
+
+    def delete(self, request, *args, **kwargs) -> HttpResponse:
+        self.object = self.get_object()
+        response = super().delete(request, *args, **kwargs)
+        self.send_success_message(_("Removed"))
+        return response
+
+    def get_success_url(self) -> str:
+        return self.object.get_absolute_url()
 
 
-class Select2SolarSystemsView(JSONResponseMixin, TemplateView):
+class Select2SolarSystemsView(JSONResponseMixin, ListView):
+    """Dynamically generated list of solar systems for select2 widget."""
+
+    model = EveSolarSystem
+
+    def get_queryset(self):
+        qs = super().get_queryset()
+        term = self.request.GET.get("term")
+        if not term:
+            return qs.none()
+        return qs.filter(name__istartswith=term)
+
+    def get_context_data(self, **kwargs):
+        if self.object_list:
+            solar_systems = self.object_list.values("id", "name")
+            results = [{"id": row["id"], "text": row["name"]} for row in solar_systems]
+            results = sorted(results, key=lambda d: d["text"])
+        else:
+            results = None
+        return {"results": results}
+
     def render_to_response(self, context, **response_kwargs):
         return self.render_to_json_response(context, **response_kwargs)
 
-    def get_context_data(self):
+
+class Select2StructureTypesView(JSONResponseMixin, ListView):
+    """Dynamically generated list of types for select2 widget."""
+
+    model = EveType
+
+    def get_queryset(self):
+        qs = super().get_queryset()
         term = self.request.GET.get("term")
-        if term:
+        if not term:
+            return qs.none()
+        qs = (
+            qs.filter(
+                eve_group__eve_category_id=EveCategoryId.STRUCTURE, published=True
+            )
+            | qs.filter(
+                eve_group_id__in=[EveGroupId.CONTROL_TOWER, EveGroupId.MOBILE_DEPOT],
+                published=True,
+            )
+            | qs.filter(
+                id__in=[EveTypeId.CUSTOMS_OFFICE, EveTypeId.IHUB, EveTypeId.TCU]
+            )
+        )
+        return qs.distinct().filter(name__icontains=term)
+
+    def get_context_data(self, **kwargs):
+        if self.object_list:
             results = [
                 {"id": row["id"], "text": row["name"]}
-                for row in EveSolarSystem.objects.filter(name__istartswith=term).values(
-                    "id", "name"
-                )
+                for row in self.object_list.values("id", "name").order_by("name")
             ]
+            results = sorted(results, key=lambda d: d["text"])
         else:
             results = None
-
         return {"results": results}
 
-
-class Select2StructureTypesView(JSONResponseMixin, TemplateView):
     def render_to_response(self, context, **response_kwargs):
         return self.render_to_json_response(context, **response_kwargs)
-
-    def get_context_data(self):
-        term = self.request.GET.get("term")
-        if term:
-            types_qs = (
-                EveType.objects.filter(
-                    eve_group__eve_category_id=EVE_CATEGORY_ID_STRUCTURE, published=True
-                )
-                | EveType.objects.filter(
-                    eve_group_id__in=[
-                        EVE_GROUP_ID_CONTROL_TOWER,
-                        EVE_GROUP_ID_MOBILE_DEPOT,
-                    ],
-                    published=True,
-                )
-                | EveType.objects.filter(
-                    id__in=[
-                        EVE_TYPE_ID_CUSTOMS_OFFICE,
-                        EVE_TYPE_ID_IHUB,
-                        EVE_TYPE_ID_TCU,
-                    ]
-                )
-            )
-            types_qs = (
-                types_qs.select_related("eve_category", "eve_category__eve_group")
-                .distinct()
-                .filter(name__icontains=term)
-            )
-            results = [
-                {"id": row["id"], "text": row["name"]}
-                for row in types_qs.values("id", "name")
-            ]
-        else:
-            results = None
-
-        return {"results": results}
